#include "NebulaClient.h"
#include "IOTCClient.h"
#include "AVAPIs.h"
#include "RDTClient.h"
#include "disposable_params.h"
#include "common.h"
#include <arpa/inet.h>
#include <ctype.h>
#include <netinet/in.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>

static char gUdid[MAX_PUBLIC_UDID_LENGTH + 1] = {0};
static char gPincode[MAX_PIN_CODE_LENGTH + 1] = {0};
static char gRentAmToken[MAX_TOKEN_LENGTH + 1] = {0};
static char gRealm[MAX_REALM_LENGTH + 1] = "hausetopia-us";

static unsigned int gBindAbort = 0;
static unsigned int gSendCommandAbort = 0;
static unsigned int gConnectAbort = 0;

static char gTestString[] = "rdtovernebula";

int main(int argc, char *argv[]) {
    int ret = 0;
    int sid = -1;
    int rdt_id = 0;
    const char *license_key = NULL;
    NebulaClientCtx *client_ctx = NULL;
    TUTKJsonObject *json_args = NULL;
    const TUTKJsonObject *json_identity = NULL;
    const char *identity_value = NULL;
    TUTKJsonObject *json_response = NULL;
    const char *profile_str = NULL;
    char buf[1024] = {0};
    int data_len = 0;

    if (argc != 3) {
        printf("%s [UDID] [AM Token]\n", argv[0]);
        return -1;
    }

    strcpy(gUdid, argv[1]);
    strcpy(gRentAmToken, argv[2]);

    ret = loadDisposableParams(gPincode, NULL, NULL);
    if (ret < 0) {
        printf("loadDisposableParams()=%d\n", ret);
        return -1;
    }

    license_key = getenv(ENV_LICENSE_KEY);
    if (license_key == NULL) {
        printf("Environment variable %s must be set!!\n", ENV_LICENSE_KEY);
        return -1;
    }

    //Set log file path
    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));

    log_attr.path = "./log_iotc_client.txt";
    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_rdt_client.txt";
    ret = RDT_Set_Log_Attr(log_attr);
    printf("RDT_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_nebula_client.txt";
    ret = Nebula_Set_Log_Attr(log_attr);
    printf("Nebula_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_global_client.txt";
    ret = TUTK_Set_Log_Attr(log_attr);
    printf("TUTK_Set_Log_Attr ret[%d]\n", ret);

    ret = TUTK_SDK_Set_License_Key(license_key);
    printf("TUTK_SDK_Set_License_Key() ret[%d]\n", ret);
    if (ret < 0) {
        return -1;
    }

    ret = Nebula_Initialize();
    printf("Nebula_Initialize() ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    ret = IOTC_Initialize2(0);
    printf("IOTC_Initialize() ret[%d]\n", ret);
    if (ret != IOTC_ER_NoERROR) {
        return -1;
    }

    ret = RDT_Initialize();
    printf("RDT_Initialize() rdt_channel[%d]\n", ret);
    if (ret < 0) {
        return -1;
    }

    ret = Nebula_Client_New(gUdid, &client_ctx);
    printf("Nebula_Client_New() ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    ret = Nebula_Client_Bind(client_ctx, gPincode, &json_args, 30000, &gBindAbort);
    printf("Nebula_Client_Bind() ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    profile_str = TUTK_Json_Obj_To_String(json_args);
    printf("[Bind_Response]\n%s\n", profile_str);

    TUTK_Json_Obj_Get_Sub_Obj(json_args, "identity", &json_identity);
    TUTK_Json_Obj_Get_String(json_identity, &identity_value);
    printf("Get identity: %s\n", identity_value);

    ret = Nebula_Client_Free_Bind_Response(json_args);
    if (ret != NEBULA_ER_NoERROR){
        printf("Nebula_Client_Free_Bind_Response() failed!\n");
    }

    char request1[] = "{\"func\":\"getProfile\"}";
    printf("get device's profile : \n%s\n", request1);
    ret = Nebula_Client_Send_Command(client_ctx, request1, &json_response, 10000, &gSendCommandAbort);
    printf("json_response [%d] \n\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    profile_str = TUTK_Json_Obj_To_String(json_response);
    printf("[getProfile]\n%s\n", profile_str);

    ret = Nebula_Client_Free_Send_Command_Response(json_response);
    if (ret != NEBULA_ER_NoERROR) {
        printf("Nebula_Client_Free_Send_Command_Response() failed!\n");
    }

    sid = IOTC_Client_Connect_By_Nebula(client_ctx, gRentAmToken, gRealm, 60000, &gConnectAbort);
    printf("IOTC_Client_Connect_By_Nebula() sid[%d]\n", sid);
    if (sid < 0) {
        return -1;
    }

    rdt_id = RDT_Client_Create(sid, 0, NULL, NULL, 3000);
    printf("RDT_Client_Create() rdt_id[%d]\n", rdt_id);
    if (ret < 0) {
        return -1;
    }

    printf("RDT send [%s]\n", gTestString);
    ret = RDT_Write(rdt_id, gTestString, strlen(gTestString));
    if (ret < 0) {
        printf("RDT_Write() ret[%d]\n", ret);
    }

    data_len = RDT_Read(rdt_id, buf, sizeof(buf), 100000);
    if (data_len < 0) {
        printf("RDT_Read() ret[%d]\n", data_len);
    }
    printf("RDT receive [%s]\n", buf);

    RDT_Destroy(rdt_id);
    RDT_DeInitialize();

    IOTC_Session_Close(sid);
    IOTC_DeInitialize();

    Nebula_Client_Delete(client_ctx);
    Nebula_DeInitialize();
    return 0;
}
